use std::{collections::HashSet, time::Duration};

use pdb_helper as pdb;
use simple_logger::SimpleLogger;
use unreal_sdk::sdk::output::rust::generate_partial_sdk_to_tmp;
use windows::s;

#[no_mangle]
pub extern "win64" fn DllMain(_module: usize, reason: u32, _: usize) -> bool {
    match reason {
        1 => {
            if std::env::current_exe()
                .expect("current exe")
                .file_name()
                .expect("file name")
                .to_string_lossy()
                == "ShooterGame.exe"
            {
                SimpleLogger::new().init().unwrap();
                log::info!("hello");
                std::thread::spawn(move || {
                    let _wnd = loop {
                        let window = unsafe {
                            windows::Win32::UI::WindowsAndMessaging::FindWindowA(
                                None,
                                s!("ARK: Survival Evolved"),
                            )
                        };
                        if window.0 != 0 {
                            log::debug!("found window: {window:?}");
                            break window;
                        }

                        log::trace!("window was 0");
                        std::thread::sleep(std::time::Duration::from_millis(150));
                    };

                    let globals = pdb::get_pdb().find_ue_globals().expect("globals");
                    unreal_sdk::global_tables::names::GNAMES
                        .write()
                        .unwrap()
                        .set_names(globals.names.cast());
                    unreal_sdk::global_tables::objects::GOBJECTS
                        .write()
                        .unwrap()
                        .set_objects(globals.objects.cast());

                    std::thread::sleep(Duration::from_secs(30));
                    let sdk = unreal_sdk::sdk::Sdk::new().patch();
                    _ = generate_partial_sdk_to_tmp(
                        &sdk,
                        HashSet::from_iter([
                            "Package_ShooterGame",
                            "Package_DeathItemCache",
                            "Package_WeapC4",
                            "Package_ExplorerChest_Base",
                            "Package_PointOfInterestBP_MissionStart",
                            "Package_BP_DedicatedStorage",
                            "Package_ElectricGenerator",
                            "Package_ExplorerChest_Base",
                            "Package_FeedingTroughBaseBP",
                            "Package_MilkGlider_Character_BP",
                            "Package_PointOfInterestBP_MissionStart",
                            "Package_SleepingBag",
                            "Package_StorageBox_Huge",
                            "Package_StorageBox_TekGenerator",
                            "Package_StructureAmmoContainer",
                            "Package_StructureTurretBaseBP_Heavy",
                            "Package_StructureTurretBaseBP",
                            "Package_StructureTurretTek",
                            "Package_Structure_DinoLeash",
                            "Package_Structure_TekAlarm",
                            "Package_BP_DedicatedStorage",
                            "Package_Buff_Companion_HLNA",
                            "Package_Buff_TekArmor_Pants",
                            "Package_Buff_TekArmor_Gloves",
                            "Package_Buff_TekArmor_Sword",
                            "Package_Buff_TekArmor_Shirt_Rework",
                        ]),
                    );
                });
            }
        }
        _ => {}
    }

    true
}
